##################################
# Snake-Spiel mit pygame in Python
##################################

# Importieren und initialisieren der Pygame-Bibliothek
import pygame
import time
import random
pygame.init()

# Konstanten setzen
green = (0, 180, 50)    # Farbe für Spielfeld
red = (220, 30, 0)      # Farbe für Essen
black = (0, 0, 0)       # Farbe für Schlange und Abbruch-Hintergrund
yellow = (240, 200, 10) # Farbe für Punkte
white = (255, 255, 255) # Farbe für Abbruch-Text

# Definieren und Öffnen eines neuen Fensters
w_width = 600
w_height = 400
window = pygame.display.set_mode((w_width, w_height))
pygame.display.set_caption("Sn4ke")
clock = pygame.time.Clock()

# Parameter für die Schlange festlegen
snake_block = 10
snake_speed = 15

# Schriftarten festlegen
font_style = pygame.font.SysFont("Arial", 20)
score_font = pygame.font.SysFont("Arial", 18, bold=True)

# Funktion für die Punktevergabe
def your_score(score):
    points = score_font.render("Punkte: " + format(score), True, yellow)
    window.blit(points, [2, 0]) # Zeichnet Points auf x=2 und y=0


# Funktion zum Zeichnen der Schlange (am Anfang ein Rechteck)
def our_snake(snake_block, snake_list):
    for x in snake_list:
        pygame.draw.rect(window, black, [x[0], x[1], snake_block, snake_block])

# Funktion für die Verloren-Nachricht
def message(msg, color):
    message = font_style.render(msg, True, color)
    window.blit(message, [w_width / 7, w_height / 3])

# Funktion für die Schleife des Hauptprogramms
def gameLoop():
    game_over = False
    game_close = False

    # Startpositionen und veränderte Koordinaten festlegen
    x1 = w_width / 2
    y1 = w_height / 2
    x1_change = 0
    y1_change = 0

    # Schlange für den Start initialisieren
    snake_list = []
    length_of_snake = 1

    # Koordinaten für Futter zufällig erstellen
    foodx = round(random.randrange(0, w_width - snake_block) / 10.0) * 10.0
    foody = round(random.randrange(0, w_height - snake_block) / 10.0) * 10.0

    # Schleife Hauptprogramm
    while not game_over:

        # Schleife wenn Abbruchbedingung erreicht wurde
        while game_close == True:
            window.fill(black)
            message("Verloren! Drücke C um fortzufahren oder Q zum Schließen", white)
            your_score(length_of_snake - 1)
            pygame.display.update()

            # Schleife die auf Eingabe zum Fortfahren oder Abbrechen wartet
            for event in pygame.event.get():
                if event.type == pygame.KEYDOWN:
                    if event.key == pygame.K_q:
                        game_over = True
                        game_close = False
                    if event.key == pygame.K_c:
                        gameLoop()

        # Schleife die auf Eingaben der Pfeiltasten wartet und die Richtung ändert
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                game_over = True
            if event.type == pygame.KEYDOWN:
                if event.key == pygame.K_LEFT:
                    x1_change = -snake_block
                    y1_change = 0
                elif event.key == pygame.K_RIGHT:
                    x1_change = snake_block
                    y1_change = 0
                elif event.key == pygame.K_UP:
                    y1_change = -snake_block
                    x1_change = 0
                elif event.key == pygame.K_DOWN:
                    y1_change = snake_block
                    x1_change = 0

        # Kollissionskontrolle mit Fensterkanten
        if x1 >= w_width or x1 < 0 or y1 >= w_height or y1 < 0:
            game_close = True

        # Fortbewegung der Schlange - Veränderung der Koordinaten
        x1 += x1_change
        y1 += y1_change

        # Spielfeld löschen und Futter zufällig positionieren
        window.fill(green)
        pygame.draw.rect(window, red, [foodx, foody, snake_block, snake_block])

        # Wachstum der Schlange nach jedem Futter
        snake_head = []
        snake_head.append(x1)
        snake_head.append(y1)
        snake_list.append(snake_head)
        if len(snake_list) > length_of_snake:
            del snake_list[0]

        # Wenn sich Schlange selbst trifft - Game Over
        for x in snake_list[:-1]:
            if x == snake_head:
                game_close = True

        # Funktionsaufruf der Schlange und Punkte
        our_snake(snake_block, snake_list)
        your_score(length_of_snake - 1)

        # Aktualisiere das Fenster nach jeder Schleifeniteration
        pygame.display.update()

        # Wenn Schlange das Essen gefunden hat
        if x1 == foodx and y1 == foody:
            foodx = round(random.randrange(0, w_width - snake_block) / 10.0) * 10.0
            foody = round(random.randrange(0, w_height - snake_block) / 10.0) * 10.0
            length_of_snake += 1

        # Wie schnell die Schleife iterieren soll (Bewegungsgeschwindigkeit)
        clock.tick(snake_speed)

    # Wenn Game Over
    pygame.quit()
    quit()

# Allgemeiner Spielaufruf
gameLoop()
